#!/usr/bin/env python2.4

## web-heartbeat.py
#
# It checks if www services are responding.
#
# Input: reads a list of URLs to be checked from heartbeat-cfg.py
#        SSL URLs will be requested using the certificate/key specified
#        in X509_HOST_CERT, X509_USER_CERT or proxy file created in /tmp
#
# Output: prints messages warning about services that are not responding.
#         optionally writes a XML to be used as SLS sensor
#
# Dependency: needs WMCore. Its sources must be in PYTHONPATH.
#
# Author: Diego da Silva Gomes (diego@cern.ch)
#
# Copyright: European Organization for Nuclear Research (CERN)
##

from urlparse import urlparse
import socket # to catch socket.error exception
import datetime
import sys
import os
from WMCore.Configuration import loadConfigurationFile
from WMCore.Services.Requests import Requests,SecureRequests


def now(utc=False):
    if utc:
        now=datetime.datetime.utcnow()
    else:
        now=datetime.datetime.now()
    return now.strftime("%Y-%m-%dT%H:%M:%S")

def queryService(url=''):
    proto,hostAndPort,path=urlparse(url)[0:3]
    if proto=='https':
        req=SecureRequests(hostAndPort)
    elif proto=='http':
        req=Requests(hostAndPort)
    else:
        print "[%s]: Invalid service: %s" % (now(),url)
        return False

    # Requests the URL
    try:
        data,status,reason=req.get(url)
    except socket.error:
        print "[%s]: Error while contacting %s: %s" % (now(),url,
                                                       sys.exc_info()[1][1])
        return False

    if (status in [404, 408]) or (status >= 500):
        print "[%s]: Error response from %s: %s - %s" % (now(),url,
                                                         status,reason)
        return False
    
    return True

# Reads and parses the input
cfg=loadConfigurationFile('@CONFIG_FILE@')
os.environ["X509_HOST_CERT"]=cfg.Heartbeat.X509_CERT
os.environ["X509_HOST_KEY"]=cfg.Heartbeat.X509_KEY
querier=True
for url in cfg.Heartbeat.urls:
    querier&=queryService(url)


# Writes the output file to SLS if it was set in the configuration
if cfg.Heartbeat.outputToSls:
    try:
        f=open(cfg.Heartbeat.outputToSls,'wt');
        xml="""<?xml version="1.0" encoding="utf-8"?>
<serviceupdate xmlns="http://sls.cern.ch/SLS/XML/update">
    <id>CMS_WEBTOOLS</id>
    <fullname>CMS Webtools Services</fullname>
    <availability>%s</availability>
    <timestamp>%s</timestamp>
    <notes>If one or more WebTools services fail, the availability will drop to zero. Otherwise it is going to be 100%%</notes>
</serviceupdate>""" % (querier*100,now(utc=True))
        f.write(xml)
        f.close()
        #2009-08-11T12:49:27+02:00
    except:
        print "[%s]: Could not write XML to %s: %s" %(now(),
                                                      cfg.Heartbeat.outputToSls,
                                                      sys.exc_info()[1][1])
# End of script

